# coding: utf-8

"""
    Gigs Core API

    API to manage voice & data plans for smartphones, wearables and IoT in 200+ countries.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Contact: support@gigs.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Optional
from pydantic import BaseModel, Field, validator

class ProjectPayments(BaseModel):
    """
    Configuration parameters for the project use of payments.
    """
    currency: Optional[Any] = Field(..., description="Three-letter ISO 4217 currency code representing the currency of the project. Must be a supported currency.")
    mode: Optional[Any] = Field(..., description="The mode in which the payments operate.")
    __properties = ["currency", "mode"]

    @validator('mode')
    def mode_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('live', 'test'):
            raise ValueError("must be one of enum values ('live', 'test')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ProjectPayments:
        """Create an instance of ProjectPayments from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if currency (nullable) is None
        # and __fields_set__ contains the field
        if self.currency is None and "currency" in self.__fields_set__:
            _dict['currency'] = None

        # set to None if mode (nullable) is None
        # and __fields_set__ contains the field
        if self.mode is None and "mode" in self.__fields_set__:
            _dict['mode'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ProjectPayments:
        """Create an instance of ProjectPayments from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ProjectPayments.parse_obj(obj)

        _obj = ProjectPayments.parse_obj({
            "currency": obj.get("currency"),
            "mode": obj.get("mode")
        })
        return _obj




# coding: utf-8

"""
    Gigs Core API

    API to manage voice & data plans for smartphones, wearables and IoT in 200+ countries.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Contact: support@gigs.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Optional
from pydantic import BaseModel, Field

class SimCredentials(BaseModel):
    """
    SimCredentials
    """
    object: Optional[Any] = Field(..., description="Type of object is always `simCredentials`.")
    activation_code: Optional[Any] = Field(..., alias="activationCode", description="eSIM activation code to download and install the profile in a device. Only available for type `eSIM`.")
    qr_code_url: Optional[Any] = Field(..., alias="qrCodeUrl", description="URL of the activation code in QR-Code format. Only available for type `eSIM`.  <!-- theme: warning -->  > Be careful how you share this URL, it can be accessed publicly without authentication.")
    sim: Optional[Any] = Field(..., description="Unique identifier for the SIM this credentials are for.")
    __properties = ["object", "activationCode", "qrCodeUrl", "sim"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SimCredentials:
        """Create an instance of SimCredentials from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if object (nullable) is None
        # and __fields_set__ contains the field
        if self.object is None and "object" in self.__fields_set__:
            _dict['object'] = None

        # set to None if activation_code (nullable) is None
        # and __fields_set__ contains the field
        if self.activation_code is None and "activation_code" in self.__fields_set__:
            _dict['activationCode'] = None

        # set to None if qr_code_url (nullable) is None
        # and __fields_set__ contains the field
        if self.qr_code_url is None and "qr_code_url" in self.__fields_set__:
            _dict['qrCodeUrl'] = None

        # set to None if sim (nullable) is None
        # and __fields_set__ contains the field
        if self.sim is None and "sim" in self.__fields_set__:
            _dict['sim'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SimCredentials:
        """Create an instance of SimCredentials from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SimCredentials.parse_obj(obj)

        _obj = SimCredentials.parse_obj({
            "object": obj.get("object"),
            "activation_code": obj.get("activationCode"),
            "qr_code_url": obj.get("qrCodeUrl"),
            "sim": obj.get("sim")
        })
        return _obj




# coding: utf-8

"""
    Gigs Core API

    API to manage voice & data plans for smartphones, wearables and IoT in 200+ countries.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Contact: support@gigs.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Optional
from pydantic import BaseModel, Field, validator

class PlanRequirements(BaseModel):
    """
    PlanRequirements
    """
    address: Optional[Any] = Field(..., description="The address requirements that must be fulfilled before a user may purchase a subscription.")
    device: Optional[Any] = Field(..., description="The device requirements that must be fulfilled before a user may purchase a subscription.")
    user_birthday: Optional[Any] = Field(..., alias="user.birthday", description="The user birthday requirements that must be fulfilled before a user may purchase a subscription.")
    user_full_name: Optional[Any] = Field(..., alias="user.fullName", description="The user full name requirements that must be fulfilled before a user may purchase a subscription.")
    __properties = ["address", "device", "user.birthday", "user.fullName"]

    @validator('address')
    def address_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('none', 'present', 'verified'):
            raise ValueError("must be one of enum values ('none', 'present', 'verified')")
        return value

    @validator('device')
    def device_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('none', 'present', 'verified'):
            raise ValueError("must be one of enum values ('none', 'present', 'verified')")
        return value

    @validator('user_birthday')
    def user_birthday_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('none', 'present', 'verified'):
            raise ValueError("must be one of enum values ('none', 'present', 'verified')")
        return value

    @validator('user_full_name')
    def user_full_name_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('none', 'present', 'verified'):
            raise ValueError("must be one of enum values ('none', 'present', 'verified')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PlanRequirements:
        """Create an instance of PlanRequirements from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if address (nullable) is None
        # and __fields_set__ contains the field
        if self.address is None and "address" in self.__fields_set__:
            _dict['address'] = None

        # set to None if device (nullable) is None
        # and __fields_set__ contains the field
        if self.device is None and "device" in self.__fields_set__:
            _dict['device'] = None

        # set to None if user_birthday (nullable) is None
        # and __fields_set__ contains the field
        if self.user_birthday is None and "user_birthday" in self.__fields_set__:
            _dict['user.birthday'] = None

        # set to None if user_full_name (nullable) is None
        # and __fields_set__ contains the field
        if self.user_full_name is None and "user_full_name" in self.__fields_set__:
            _dict['user.fullName'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PlanRequirements:
        """Create an instance of PlanRequirements from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PlanRequirements.parse_obj(obj)

        _obj = PlanRequirements.parse_obj({
            "address": obj.get("address"),
            "device": obj.get("device"),
            "user_birthday": obj.get("user.birthday"),
            "user_full_name": obj.get("user.fullName")
        })
        return _obj




# coding: utf-8

"""
    Gigs Core API

    API to manage voice & data plans for smartphones, wearables and IoT in 200+ countries.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Contact: support@gigs.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Optional
from pydantic import BaseModel, Field

class PortingsCreateRequest(BaseModel):
    """
    PortingsCreateRequest
    """
    account_number: Optional[Any] = Field(None, alias="accountNumber", description="The account number on the donor service provider.")
    account_pin: Optional[Any] = Field(None, alias="accountPin", description="The pin for the account on the donor service provider. This will be used only for the porting.")
    birthday: Optional[Any] = Field(None, description="The birthday of the account holder on the donor service provider.")
    donor_provider: Optional[Any] = Field(None, alias="donorProvider", description="The unique identifier of the donor service provider.")
    first_name: Optional[Any] = Field(None, alias="firstName", description="The first name of the account holder.")
    last_name: Optional[Any] = Field(None, alias="lastName", description="The last name of the account holder.")
    phone_number: Optional[Any] = Field(..., alias="phoneNumber", description="The phone number to port in E.164 format.")
    provider: Optional[Any] = Field(..., description="The name of the receiving network provider.")
    user: Optional[Any] = Field(..., description="The unique identifier for an existing [user](https://developers.gigs.com/docs/api/b3A6MzMwODcxNDM-retrieve-a-user) the porting should be created for.")
    __properties = ["accountNumber", "accountPin", "birthday", "donorProvider", "firstName", "lastName", "phoneNumber", "provider", "user"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PortingsCreateRequest:
        """Create an instance of PortingsCreateRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if account_number (nullable) is None
        # and __fields_set__ contains the field
        if self.account_number is None and "account_number" in self.__fields_set__:
            _dict['accountNumber'] = None

        # set to None if account_pin (nullable) is None
        # and __fields_set__ contains the field
        if self.account_pin is None and "account_pin" in self.__fields_set__:
            _dict['accountPin'] = None

        # set to None if birthday (nullable) is None
        # and __fields_set__ contains the field
        if self.birthday is None and "birthday" in self.__fields_set__:
            _dict['birthday'] = None

        # set to None if donor_provider (nullable) is None
        # and __fields_set__ contains the field
        if self.donor_provider is None and "donor_provider" in self.__fields_set__:
            _dict['donorProvider'] = None

        # set to None if first_name (nullable) is None
        # and __fields_set__ contains the field
        if self.first_name is None and "first_name" in self.__fields_set__:
            _dict['firstName'] = None

        # set to None if last_name (nullable) is None
        # and __fields_set__ contains the field
        if self.last_name is None and "last_name" in self.__fields_set__:
            _dict['lastName'] = None

        # set to None if phone_number (nullable) is None
        # and __fields_set__ contains the field
        if self.phone_number is None and "phone_number" in self.__fields_set__:
            _dict['phoneNumber'] = None

        # set to None if provider (nullable) is None
        # and __fields_set__ contains the field
        if self.provider is None and "provider" in self.__fields_set__:
            _dict['provider'] = None

        # set to None if user (nullable) is None
        # and __fields_set__ contains the field
        if self.user is None and "user" in self.__fields_set__:
            _dict['user'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PortingsCreateRequest:
        """Create an instance of PortingsCreateRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PortingsCreateRequest.parse_obj(obj)

        _obj = PortingsCreateRequest.parse_obj({
            "account_number": obj.get("accountNumber"),
            "account_pin": obj.get("accountPin"),
            "birthday": obj.get("birthday"),
            "donor_provider": obj.get("donorProvider"),
            "first_name": obj.get("firstName"),
            "last_name": obj.get("lastName"),
            "phone_number": obj.get("phoneNumber"),
            "provider": obj.get("provider"),
            "user": obj.get("user")
        })
        return _obj




# coding: utf-8

"""
    Gigs Core API

    API to manage voice & data plans for smartphones, wearables and IoT in 200+ countries.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Contact: support@gigs.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Optional
from pydantic import BaseModel, Field, validator
from gigs_client.models.porting_donor_provider import PortingDonorProvider
from gigs_client.models.porting_recipient_provider import PortingRecipientProvider

class SubscriptionPorting(BaseModel):
    """
    Current porting (if any) related with the subscription.
    """
    object: Optional[Any] = Field(..., description="Type of object is always `porting`.")
    id: Optional[Any] = Field(..., description="Unique identifier for the porting.")
    account_number: Optional[Any] = Field(..., alias="accountNumber", description="The account number on the donor service provider.")
    birthday: Optional[Any] = Field(..., description="The birthday of the account holder on the donor service provider.")
    declined_attempts: Optional[Any] = Field(..., alias="declinedAttempts", description="The number of declined porting attempts so far.")
    declined_code: Optional[Any] = Field(..., alias="declinedCode", description="The code reason for the declined porting.  <!-- theme: info --> > #### Preview > > This property is currently in preview and might change in the future. > We’re excited to hear your feedback and ideas. Please send an email > to [support@gigs.com](mailto:support@gigs.com) to share your thoughts. ")
    declined_message: Optional[Any] = Field(..., alias="declinedMessage", description="The reason description for the declined porting.  <!-- theme: info --> > #### Preview > > This property is currently in preview and might change in the future. > We’re excited to hear your feedback and ideas. Please send an email > to [support@gigs.com](mailto:support@gigs.com) to share your thoughts. ")
    donor_provider: PortingDonorProvider = Field(..., alias="donorProvider")
    first_name: Optional[Any] = Field(..., alias="firstName", description="The first name of the account holder on the donor service provider.")
    last_name: Optional[Any] = Field(..., alias="lastName", description="The last name of the account holder on the donor service provider.")
    phone_number: Optional[Any] = Field(..., alias="phoneNumber", description="The phone number to port in E.164 format.")
    provider: Optional[Any] = Field(..., description="The ID of the recipient network provider.")
    recipient_provider: PortingRecipientProvider = Field(..., alias="recipientProvider")
    required: Optional[Any] = Field(..., description="List of fields required for the porting to be processable. The fields depend on the donor and recipient providers.")
    status: Optional[Any] = Field(..., description="The current status of the porting.")
    subscription: Optional[Any] = Field(..., description="Unique identifier for the porting subscription.")
    user: Optional[Any] = Field(..., description="Unique identifier for the user this porting is for.")
    canceled_at: Optional[Any] = Field(..., alias="canceledAt", description="Time when the porting was canceled.")
    completed_at: Optional[Any] = Field(..., alias="completedAt", description="Time when the porting was completed.")
    created_at: Optional[Any] = Field(..., alias="createdAt", description="Time when the porting was created.")
    expired_at: Optional[Any] = Field(..., alias="expiredAt", description="Time when the porting was expired.")
    last_declined_at: Optional[Any] = Field(..., alias="lastDeclinedAt", description="Time when the porting was last declined.")
    last_requested_at: Optional[Any] = Field(..., alias="lastRequestedAt", description="Time when the porting was last requested.")
    __properties = ["object", "id", "accountNumber", "birthday", "declinedAttempts", "declinedCode", "declinedMessage", "donorProvider", "firstName", "lastName", "phoneNumber", "provider", "recipientProvider", "required", "status", "subscription", "user", "canceledAt", "completedAt", "createdAt", "expiredAt", "lastDeclinedAt", "lastRequestedAt"]

    @validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('draft', 'pending', 'requested', 'declined', 'completed', 'canceled', 'expired'):
            raise ValueError("must be one of enum values ('draft', 'pending', 'requested', 'declined', 'completed', 'canceled', 'expired')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SubscriptionPorting:
        """Create an instance of SubscriptionPorting from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of donor_provider
        if self.donor_provider:
            _dict['donorProvider'] = self.donor_provider.to_dict()
        # override the default output from pydantic by calling `to_dict()` of recipient_provider
        if self.recipient_provider:
            _dict['recipientProvider'] = self.recipient_provider.to_dict()
        # set to None if object (nullable) is None
        # and __fields_set__ contains the field
        if self.object is None and "object" in self.__fields_set__:
            _dict['object'] = None

        # set to None if id (nullable) is None
        # and __fields_set__ contains the field
        if self.id is None and "id" in self.__fields_set__:
            _dict['id'] = None

        # set to None if account_number (nullable) is None
        # and __fields_set__ contains the field
        if self.account_number is None and "account_number" in self.__fields_set__:
            _dict['accountNumber'] = None

        # set to None if birthday (nullable) is None
        # and __fields_set__ contains the field
        if self.birthday is None and "birthday" in self.__fields_set__:
            _dict['birthday'] = None

        # set to None if declined_attempts (nullable) is None
        # and __fields_set__ contains the field
        if self.declined_attempts is None and "declined_attempts" in self.__fields_set__:
            _dict['declinedAttempts'] = None

        # set to None if declined_code (nullable) is None
        # and __fields_set__ contains the field
        if self.declined_code is None and "declined_code" in self.__fields_set__:
            _dict['declinedCode'] = None

        # set to None if declined_message (nullable) is None
        # and __fields_set__ contains the field
        if self.declined_message is None and "declined_message" in self.__fields_set__:
            _dict['declinedMessage'] = None

        # set to None if first_name (nullable) is None
        # and __fields_set__ contains the field
        if self.first_name is None and "first_name" in self.__fields_set__:
            _dict['firstName'] = None

        # set to None if last_name (nullable) is None
        # and __fields_set__ contains the field
        if self.last_name is None and "last_name" in self.__fields_set__:
            _dict['lastName'] = None

        # set to None if phone_number (nullable) is None
        # and __fields_set__ contains the field
        if self.phone_number is None and "phone_number" in self.__fields_set__:
            _dict['phoneNumber'] = None

        # set to None if provider (nullable) is None
        # and __fields_set__ contains the field
        if self.provider is None and "provider" in self.__fields_set__:
            _dict['provider'] = None

        # set to None if required (nullable) is None
        # and __fields_set__ contains the field
        if self.required is None and "required" in self.__fields_set__:
            _dict['required'] = None

        # set to None if status (nullable) is None
        # and __fields_set__ contains the field
        if self.status is None and "status" in self.__fields_set__:
            _dict['status'] = None

        # set to None if subscription (nullable) is None
        # and __fields_set__ contains the field
        if self.subscription is None and "subscription" in self.__fields_set__:
            _dict['subscription'] = None

        # set to None if user (nullable) is None
        # and __fields_set__ contains the field
        if self.user is None and "user" in self.__fields_set__:
            _dict['user'] = None

        # set to None if canceled_at (nullable) is None
        # and __fields_set__ contains the field
        if self.canceled_at is None and "canceled_at" in self.__fields_set__:
            _dict['canceledAt'] = None

        # set to None if completed_at (nullable) is None
        # and __fields_set__ contains the field
        if self.completed_at is None and "completed_at" in self.__fields_set__:
            _dict['completedAt'] = None

        # set to None if created_at (nullable) is None
        # and __fields_set__ contains the field
        if self.created_at is None and "created_at" in self.__fields_set__:
            _dict['createdAt'] = None

        # set to None if expired_at (nullable) is None
        # and __fields_set__ contains the field
        if self.expired_at is None and "expired_at" in self.__fields_set__:
            _dict['expiredAt'] = None

        # set to None if last_declined_at (nullable) is None
        # and __fields_set__ contains the field
        if self.last_declined_at is None and "last_declined_at" in self.__fields_set__:
            _dict['lastDeclinedAt'] = None

        # set to None if last_requested_at (nullable) is None
        # and __fields_set__ contains the field
        if self.last_requested_at is None and "last_requested_at" in self.__fields_set__:
            _dict['lastRequestedAt'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SubscriptionPorting:
        """Create an instance of SubscriptionPorting from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SubscriptionPorting.parse_obj(obj)

        _obj = SubscriptionPorting.parse_obj({
            "object": obj.get("object"),
            "id": obj.get("id"),
            "account_number": obj.get("accountNumber"),
            "birthday": obj.get("birthday"),
            "declined_attempts": obj.get("declinedAttempts"),
            "declined_code": obj.get("declinedCode"),
            "declined_message": obj.get("declinedMessage"),
            "donor_provider": PortingDonorProvider.from_dict(obj.get("donorProvider")) if obj.get("donorProvider") is not None else None,
            "first_name": obj.get("firstName"),
            "last_name": obj.get("lastName"),
            "phone_number": obj.get("phoneNumber"),
            "provider": obj.get("provider"),
            "recipient_provider": PortingRecipientProvider.from_dict(obj.get("recipientProvider")) if obj.get("recipientProvider") is not None else None,
            "required": obj.get("required"),
            "status": obj.get("status"),
            "subscription": obj.get("subscription"),
            "user": obj.get("user"),
            "canceled_at": obj.get("canceledAt"),
            "completed_at": obj.get("completedAt"),
            "created_at": obj.get("createdAt"),
            "expired_at": obj.get("expiredAt"),
            "last_declined_at": obj.get("lastDeclinedAt"),
            "last_requested_at": obj.get("lastRequestedAt")
        })
        return _obj




# coding: utf-8

"""
    Gigs Core API

    API to manage voice & data plans for smartphones, wearables and IoT in 200+ countries.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Contact: support@gigs.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Optional
from pydantic import BaseModel, Field, validator
from gigs_client.models.coverage import Coverage
from gigs_client.models.plan_requirements import PlanRequirements
from gigs_client.models.price import Price
from gigs_client.models.validity import Validity

class Plan(BaseModel):
    """
    Plan
    """
    object: Optional[Any] = Field(..., description="Type of object is always `plan`.")
    id: Optional[Any] = Field(..., description="Unique identifier for the plan.")
    coverage: Coverage = Field(...)
    data: Optional[Any] = Field(..., description="The data allowance included in this plan. A value of `-1` indicates unlimited data.")
    data_unit: Optional[Any] = Field(..., alias="dataUnit", description="The unit of the data value is always `byte`.")
    description: Optional[Any] = Field(..., description="The plan's description, meant to be displayable to the users.")
    image: Optional[Any] = Field(..., description="The plan's image, meant to be displayable to the users.")
    name: Optional[Any] = Field(..., description="The plan's name, meant to be displayable to the users.")
    price: Price = Field(...)
    provider: Optional[Any] = Field(..., description="The ID of the network provider supported by this plan.")
    requirements: PlanRequirements = Field(...)
    sim_types: Optional[Any] = Field(..., alias="simTypes", description="List of SIM types supported by this plan. Can be `eSIM`, `pSIM`, or both.")
    sms: Optional[Any] = Field(..., description="The SMS allowance included in this plan. A value of `-1` indicates unlimited SMS.")
    sms_unit: Optional[Any] = Field(..., alias="smsUnit", description="The unit of the sms value is always `message`.")
    status: Optional[Any] = Field(...)
    validity: Validity = Field(...)
    voice: Optional[Any] = Field(..., description="The voice call allowance in seconds included in this plan. A value of `-1` indicates unlimited voice calls.")
    voice_unit: Optional[Any] = Field(..., alias="voiceUnit", description="The unit of the voice value is always `second`.")
    created_at: Optional[Any] = Field(..., alias="createdAt", description="Time when the plan was created.")
    __properties = ["object", "id", "coverage", "data", "dataUnit", "description", "image", "name", "price", "provider", "requirements", "simTypes", "sms", "smsUnit", "status", "validity", "voice", "voiceUnit", "createdAt"]

    @validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('available', 'archived', 'pending', 'draft'):
            raise ValueError("must be one of enum values ('available', 'archived', 'pending', 'draft')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Plan:
        """Create an instance of Plan from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of coverage
        if self.coverage:
            _dict['coverage'] = self.coverage.to_dict()
        # override the default output from pydantic by calling `to_dict()` of price
        if self.price:
            _dict['price'] = self.price.to_dict()
        # override the default output from pydantic by calling `to_dict()` of requirements
        if self.requirements:
            _dict['requirements'] = self.requirements.to_dict()
        # override the default output from pydantic by calling `to_dict()` of validity
        if self.validity:
            _dict['validity'] = self.validity.to_dict()
        # set to None if object (nullable) is None
        # and __fields_set__ contains the field
        if self.object is None and "object" in self.__fields_set__:
            _dict['object'] = None

        # set to None if id (nullable) is None
        # and __fields_set__ contains the field
        if self.id is None and "id" in self.__fields_set__:
            _dict['id'] = None

        # set to None if data (nullable) is None
        # and __fields_set__ contains the field
        if self.data is None and "data" in self.__fields_set__:
            _dict['data'] = None

        # set to None if data_unit (nullable) is None
        # and __fields_set__ contains the field
        if self.data_unit is None and "data_unit" in self.__fields_set__:
            _dict['dataUnit'] = None

        # set to None if description (nullable) is None
        # and __fields_set__ contains the field
        if self.description is None and "description" in self.__fields_set__:
            _dict['description'] = None

        # set to None if image (nullable) is None
        # and __fields_set__ contains the field
        if self.image is None and "image" in self.__fields_set__:
            _dict['image'] = None

        # set to None if name (nullable) is None
        # and __fields_set__ contains the field
        if self.name is None and "name" in self.__fields_set__:
            _dict['name'] = None

        # set to None if provider (nullable) is None
        # and __fields_set__ contains the field
        if self.provider is None and "provider" in self.__fields_set__:
            _dict['provider'] = None

        # set to None if sim_types (nullable) is None
        # and __fields_set__ contains the field
        if self.sim_types is None and "sim_types" in self.__fields_set__:
            _dict['simTypes'] = None

        # set to None if sms (nullable) is None
        # and __fields_set__ contains the field
        if self.sms is None and "sms" in self.__fields_set__:
            _dict['sms'] = None

        # set to None if sms_unit (nullable) is None
        # and __fields_set__ contains the field
        if self.sms_unit is None and "sms_unit" in self.__fields_set__:
            _dict['smsUnit'] = None

        # set to None if status (nullable) is None
        # and __fields_set__ contains the field
        if self.status is None and "status" in self.__fields_set__:
            _dict['status'] = None

        # set to None if voice (nullable) is None
        # and __fields_set__ contains the field
        if self.voice is None and "voice" in self.__fields_set__:
            _dict['voice'] = None

        # set to None if voice_unit (nullable) is None
        # and __fields_set__ contains the field
        if self.voice_unit is None and "voice_unit" in self.__fields_set__:
            _dict['voiceUnit'] = None

        # set to None if created_at (nullable) is None
        # and __fields_set__ contains the field
        if self.created_at is None and "created_at" in self.__fields_set__:
            _dict['createdAt'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> Plan:
        """Create an instance of Plan from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return Plan.parse_obj(obj)

        _obj = Plan.parse_obj({
            "object": obj.get("object"),
            "id": obj.get("id"),
            "coverage": Coverage.from_dict(obj.get("coverage")) if obj.get("coverage") is not None else None,
            "data": obj.get("data"),
            "data_unit": obj.get("dataUnit"),
            "description": obj.get("description"),
            "image": obj.get("image"),
            "name": obj.get("name"),
            "price": Price.from_dict(obj.get("price")) if obj.get("price") is not None else None,
            "provider": obj.get("provider"),
            "requirements": PlanRequirements.from_dict(obj.get("requirements")) if obj.get("requirements") is not None else None,
            "sim_types": obj.get("simTypes"),
            "sms": obj.get("sms"),
            "sms_unit": obj.get("smsUnit"),
            "status": obj.get("status"),
            "validity": Validity.from_dict(obj.get("validity")) if obj.get("validity") is not None else None,
            "voice": obj.get("voice"),
            "voice_unit": obj.get("voiceUnit"),
            "created_at": obj.get("createdAt")
        })
        return _obj




# coding: utf-8

"""
    Gigs Core API

    API to manage voice & data plans for smartphones, wearables and IoT in 200+ countries.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Contact: support@gigs.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import re  # noqa: F401
import io
import warnings

from pydantic import validate_arguments, ValidationError
from typing_extensions import Annotated

from pydantic import Field

from typing import Any, Optional

from gigs_client.models.porting import Porting
from gigs_client.models.portings_create_request import PortingsCreateRequest
from gigs_client.models.portings_list200_response import PortingsList200Response
from gigs_client.models.portings_update_request import PortingsUpdateRequest
from gigs_client.models.service_provider import ServiceProvider
from gigs_client.models.service_providers_list200_response import ServiceProvidersList200Response

from gigs_client.api_client import ApiClient
from gigs_client.api_response import ApiResponse
from gigs_client.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class PortingsApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def portings_cancel(self, project : Annotated[Any, Field(..., description="The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project).")], id : Annotated[Any, Field(..., description="The unique identifier for the porting.")], **kwargs) -> Porting:  # noqa: E501
        """Cancel a porting  # noqa: E501

        Retrieves the details of an existing porting and cancels it. Only portings that have not yet being completed can be canceled.  <!-- theme: info --> > #### Preview > > This endpoint is currently in preview and might change in the future. > We’re excited to hear your feedback and ideas. Please send an email > to [support@gigs.com](mailto:support@gigs.com) to share your thoughts.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.portings_cancel(project, id, async_req=True)
        >>> result = thread.get()

        :param project: The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project). (required)
        :type project: object
        :param id: The unique identifier for the porting. (required)
        :type id: object
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Porting
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the portings_cancel_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.portings_cancel_with_http_info(project, id, **kwargs)  # noqa: E501

    @validate_arguments
    def portings_cancel_with_http_info(self, project : Annotated[Any, Field(..., description="The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project).")], id : Annotated[Any, Field(..., description="The unique identifier for the porting.")], **kwargs) -> ApiResponse:  # noqa: E501
        """Cancel a porting  # noqa: E501

        Retrieves the details of an existing porting and cancels it. Only portings that have not yet being completed can be canceled.  <!-- theme: info --> > #### Preview > > This endpoint is currently in preview and might change in the future. > We’re excited to hear your feedback and ideas. Please send an email > to [support@gigs.com](mailto:support@gigs.com) to share your thoughts.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.portings_cancel_with_http_info(project, id, async_req=True)
        >>> result = thread.get()

        :param project: The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project). (required)
        :type project: object
        :param id: The unique identifier for the porting. (required)
        :type id: object
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Porting, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'project',
            'id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method portings_cancel" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['project']:
            _path_params['project'] = _params['project']

        if _params['id']:
            _path_params['id'] = _params['id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "Porting",
            '404': None,
            '422': None,
        }

        return self.api_client.call_api(
            '/projects/{project}/portings/{id}/cancel', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def portings_create(self, project : Annotated[Any, Field(..., description="The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project).")], portings_create_request : Annotated[PortingsCreateRequest, Field(..., description="Porting parameters to create.")], **kwargs) -> Porting:  # noqa: E501
        """Create a porting  # noqa: E501

        Creates a new porting with the parameters passed. The porting will be created in `draft` status and can be later used to create an order or subscription for the given network provider.  Only some of the fields are required during creation creation, but all of the ones required by the network provider must be set before submitting the porting.   <!-- theme: info --> > #### Preview > > This endpoint is currently in preview and might change in the future. > We’re excited to hear your feedback and ideas. Please send an email > to [support@gigs.com](mailto:support@gigs.com) to share your thoughts.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.portings_create(project, portings_create_request, async_req=True)
        >>> result = thread.get()

        :param project: The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project). (required)
        :type project: object
        :param portings_create_request: Porting parameters to create. (required)
        :type portings_create_request: PortingsCreateRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Porting
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the portings_create_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.portings_create_with_http_info(project, portings_create_request, **kwargs)  # noqa: E501

    @validate_arguments
    def portings_create_with_http_info(self, project : Annotated[Any, Field(..., description="The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project).")], portings_create_request : Annotated[PortingsCreateRequest, Field(..., description="Porting parameters to create.")], **kwargs) -> ApiResponse:  # noqa: E501
        """Create a porting  # noqa: E501

        Creates a new porting with the parameters passed. The porting will be created in `draft` status and can be later used to create an order or subscription for the given network provider.  Only some of the fields are required during creation creation, but all of the ones required by the network provider must be set before submitting the porting.   <!-- theme: info --> > #### Preview > > This endpoint is currently in preview and might change in the future. > We’re excited to hear your feedback and ideas. Please send an email > to [support@gigs.com](mailto:support@gigs.com) to share your thoughts.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.portings_create_with_http_info(project, portings_create_request, async_req=True)
        >>> result = thread.get()

        :param project: The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project). (required)
        :type project: object
        :param portings_create_request: Porting parameters to create. (required)
        :type portings_create_request: PortingsCreateRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Porting, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'project',
            'portings_create_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method portings_create" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['project']:
            _path_params['project'] = _params['project']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['portings_create_request'] is not None:
            _body_params = _params['portings_create_request']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '201': "Porting",
            '422': None,
        }

        return self.api_client.call_api(
            '/projects/{project}/portings', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def portings_list(self, after : Annotated[Optional[Any], Field(description="A cursor for use in pagination. The `after` parameter takes an object ID that defines the position in the list, only items immediately following the item with that ID will be returned.")] = None, before : Annotated[Optional[Any], Field(description="A cursor for use in pagination. The `before` parameter takes an object ID that defines the position in the list, only items immediately preceding the item with that ID will be returned.")] = None, limit : Annotated[Optional[Any], Field(description="The limit of items to be returned in the list, between 0 and 200.")] = None, **kwargs) -> PortingsList200Response:  # noqa: E501
        """List all portings  # noqa: E501

        Returns a list of portings. The portings returned are sorted by creation date, with the most recently created portings appearing first.  <!-- theme: info --> > #### Preview > > This endpoint is currently in preview and might change in the future. > We’re excited to hear your feedback and ideas. Please send an email > to [support@gigs.com](mailto:support@gigs.com) to share your thoughts.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.portings_list(after, before, limit, async_req=True)
        >>> result = thread.get()

        :param after: A cursor for use in pagination. The `after` parameter takes an object ID that defines the position in the list, only items immediately following the item with that ID will be returned.
        :type after: object
        :param before: A cursor for use in pagination. The `before` parameter takes an object ID that defines the position in the list, only items immediately preceding the item with that ID will be returned.
        :type before: object
        :param limit: The limit of items to be returned in the list, between 0 and 200.
        :type limit: object
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PortingsList200Response
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the portings_list_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.portings_list_with_http_info(after, before, limit, **kwargs)  # noqa: E501

    @validate_arguments
    def portings_list_with_http_info(self, after : Annotated[Optional[Any], Field(description="A cursor for use in pagination. The `after` parameter takes an object ID that defines the position in the list, only items immediately following the item with that ID will be returned.")] = None, before : Annotated[Optional[Any], Field(description="A cursor for use in pagination. The `before` parameter takes an object ID that defines the position in the list, only items immediately preceding the item with that ID will be returned.")] = None, limit : Annotated[Optional[Any], Field(description="The limit of items to be returned in the list, between 0 and 200.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """List all portings  # noqa: E501

        Returns a list of portings. The portings returned are sorted by creation date, with the most recently created portings appearing first.  <!-- theme: info --> > #### Preview > > This endpoint is currently in preview and might change in the future. > We’re excited to hear your feedback and ideas. Please send an email > to [support@gigs.com](mailto:support@gigs.com) to share your thoughts.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.portings_list_with_http_info(after, before, limit, async_req=True)
        >>> result = thread.get()

        :param after: A cursor for use in pagination. The `after` parameter takes an object ID that defines the position in the list, only items immediately following the item with that ID will be returned.
        :type after: object
        :param before: A cursor for use in pagination. The `before` parameter takes an object ID that defines the position in the list, only items immediately preceding the item with that ID will be returned.
        :type before: object
        :param limit: The limit of items to be returned in the list, between 0 and 200.
        :type limit: object
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PortingsList200Response, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'after',
            'before',
            'limit'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method portings_list" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('after') is not None:  # noqa: E501
            _query_params.append(('after', _params['after']))

        if _params.get('before') is not None:  # noqa: E501
            _query_params.append(('before', _params['before']))

        if _params.get('limit') is not None:  # noqa: E501
            _query_params.append(('limit', _params['limit']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "PortingsList200Response",
            '422': None,
        }

        return self.api_client.call_api(
            '/projects/{project}/portings', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def portings_retrieve(self, project : Annotated[Any, Field(..., description="The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project).")], id : Annotated[Any, Field(..., description="The unique identifier for the porting.")], **kwargs) -> Porting:  # noqa: E501
        """Retrieve a Porting  # noqa: E501

        Retrieves the details of an existing porting.  <!-- theme: info --> > #### Preview > > This endpoint is currently in preview and might change in the future. > We’re excited to hear your feedback and ideas. Please send an email > to [support@gigs.com](mailto:support@gigs.com) to share your thoughts.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.portings_retrieve(project, id, async_req=True)
        >>> result = thread.get()

        :param project: The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project). (required)
        :type project: object
        :param id: The unique identifier for the porting. (required)
        :type id: object
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Porting
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the portings_retrieve_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.portings_retrieve_with_http_info(project, id, **kwargs)  # noqa: E501

    @validate_arguments
    def portings_retrieve_with_http_info(self, project : Annotated[Any, Field(..., description="The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project).")], id : Annotated[Any, Field(..., description="The unique identifier for the porting.")], **kwargs) -> ApiResponse:  # noqa: E501
        """Retrieve a Porting  # noqa: E501

        Retrieves the details of an existing porting.  <!-- theme: info --> > #### Preview > > This endpoint is currently in preview and might change in the future. > We’re excited to hear your feedback and ideas. Please send an email > to [support@gigs.com](mailto:support@gigs.com) to share your thoughts.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.portings_retrieve_with_http_info(project, id, async_req=True)
        >>> result = thread.get()

        :param project: The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project). (required)
        :type project: object
        :param id: The unique identifier for the porting. (required)
        :type id: object
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Porting, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'project',
            'id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method portings_retrieve" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['project']:
            _path_params['project'] = _params['project']

        if _params['id']:
            _path_params['id'] = _params['id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "Porting",
            '404': None,
        }

        return self.api_client.call_api(
            '/projects/{project}/portings/{id}', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def portings_update(self, project : Annotated[Any, Field(..., description="The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project).")], id : Annotated[Any, Field(..., description="The unique identifier for the porting.")], portings_update_request : Annotated[PortingsUpdateRequest, Field(..., description="Porting parameters to update.")], **kwargs) -> Porting:  # noqa: E501
        """Update a porting  # noqa: E501

        Updates the specified porting by setting the values of the parameters passed.  <!-- theme: info --> > #### Preview > > This endpoint is currently in preview and might change in the future. > We’re excited to hear your feedback and ideas. Please send an email > to [support@gigs.com](mailto:support@gigs.com) to share your thoughts.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.portings_update(project, id, portings_update_request, async_req=True)
        >>> result = thread.get()

        :param project: The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project). (required)
        :type project: object
        :param id: The unique identifier for the porting. (required)
        :type id: object
        :param portings_update_request: Porting parameters to update. (required)
        :type portings_update_request: PortingsUpdateRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Porting
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the portings_update_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.portings_update_with_http_info(project, id, portings_update_request, **kwargs)  # noqa: E501

    @validate_arguments
    def portings_update_with_http_info(self, project : Annotated[Any, Field(..., description="The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project).")], id : Annotated[Any, Field(..., description="The unique identifier for the porting.")], portings_update_request : Annotated[PortingsUpdateRequest, Field(..., description="Porting parameters to update.")], **kwargs) -> ApiResponse:  # noqa: E501
        """Update a porting  # noqa: E501

        Updates the specified porting by setting the values of the parameters passed.  <!-- theme: info --> > #### Preview > > This endpoint is currently in preview and might change in the future. > We’re excited to hear your feedback and ideas. Please send an email > to [support@gigs.com](mailto:support@gigs.com) to share your thoughts.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.portings_update_with_http_info(project, id, portings_update_request, async_req=True)
        >>> result = thread.get()

        :param project: The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project). (required)
        :type project: object
        :param id: The unique identifier for the porting. (required)
        :type id: object
        :param portings_update_request: Porting parameters to update. (required)
        :type portings_update_request: PortingsUpdateRequest
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Porting, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'project',
            'id',
            'portings_update_request'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method portings_update" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['project']:
            _path_params['project'] = _params['project']

        if _params['id']:
            _path_params['id'] = _params['id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['portings_update_request'] is not None:
            _body_params = _params['portings_update_request']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "Porting",
            '404': None,
            '422': None,
        }

        return self.api_client.call_api(
            '/projects/{project}/portings/{id}', 'PATCH',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def service_providers_list(self, recipient_provider : Annotated[Optional[Any], Field(description="Only return service providers that have the given network provider as supported porting recipient.")] = None, **kwargs) -> ServiceProvidersList200Response:  # noqa: E501
        """List all service providers  # noqa: E501

        Returns a list of service providers.  <!-- theme: info --> > #### Preview > > This endpoint is currently in preview and might change in the future. > We’re excited to hear your feedback and ideas. Please send an email > to [support@gigs.com](mailto:support@gigs.com) to share your thoughts.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.service_providers_list(recipient_provider, async_req=True)
        >>> result = thread.get()

        :param recipient_provider: Only return service providers that have the given network provider as supported porting recipient.
        :type recipient_provider: object
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: ServiceProvidersList200Response
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the service_providers_list_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.service_providers_list_with_http_info(recipient_provider, **kwargs)  # noqa: E501

    @validate_arguments
    def service_providers_list_with_http_info(self, recipient_provider : Annotated[Optional[Any], Field(description="Only return service providers that have the given network provider as supported porting recipient.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """List all service providers  # noqa: E501

        Returns a list of service providers.  <!-- theme: info --> > #### Preview > > This endpoint is currently in preview and might change in the future. > We’re excited to hear your feedback and ideas. Please send an email > to [support@gigs.com](mailto:support@gigs.com) to share your thoughts.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.service_providers_list_with_http_info(recipient_provider, async_req=True)
        >>> result = thread.get()

        :param recipient_provider: Only return service providers that have the given network provider as supported porting recipient.
        :type recipient_provider: object
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(ServiceProvidersList200Response, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'recipient_provider'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method service_providers_list" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('recipient_provider') is not None:  # noqa: E501
            _query_params.append(('recipientProvider', _params['recipient_provider']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "ServiceProvidersList200Response",
            '422': None,
        }

        return self.api_client.call_api(
            '/serviceProviders', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def service_providers_retrieve(self, id : Annotated[Any, Field(..., description="The unique identifier for the service provider.")], **kwargs) -> ServiceProvider:  # noqa: E501
        """Retrieve a service provider  # noqa: E501

        Retrieves the details of a service provider.  <!-- theme: info --> > #### Preview > > This endpoint is currently in preview and might change in the future. > We’re excited to hear your feedback and ideas. Please send an email > to [support@gigs.com](mailto:support@gigs.com) to share your thoughts.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.service_providers_retrieve(id, async_req=True)
        >>> result = thread.get()

        :param id: The unique identifier for the service provider. (required)
        :type id: object
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: ServiceProvider
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the service_providers_retrieve_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.service_providers_retrieve_with_http_info(id, **kwargs)  # noqa: E501

    @validate_arguments
    def service_providers_retrieve_with_http_info(self, id : Annotated[Any, Field(..., description="The unique identifier for the service provider.")], **kwargs) -> ApiResponse:  # noqa: E501
        """Retrieve a service provider  # noqa: E501

        Retrieves the details of a service provider.  <!-- theme: info --> > #### Preview > > This endpoint is currently in preview and might change in the future. > We’re excited to hear your feedback and ideas. Please send an email > to [support@gigs.com](mailto:support@gigs.com) to share your thoughts.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.service_providers_retrieve_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param id: The unique identifier for the service provider. (required)
        :type id: object
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(ServiceProvider, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method service_providers_retrieve" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['id']:
            _path_params['id'] = _params['id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "ServiceProvider",
            '404': None,
        }

        return self.api_client.call_api(
            '/serviceProviders/{id}', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

"""
    Gigs Core API

    API to manage voice & data plans for smartphones, wearables and IoT in 200+ countries.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Contact: support@gigs.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Optional
from pydantic import BaseModel, Field

class SimsSearchRequest(BaseModel):
    """
    SimsSearchRequest
    """
    iccid: Optional[Any] = Field(..., description="The ICCID (integrated circuit card identifier) associated with the SIM.")
    type: Optional[Any] = Field(None, description="Optional type of the SIM card, either `eSIM`, `pSIM` or both.")
    __properties = ["iccid", "type"]

    @validator('iccid')
    def iccid_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^([0-9]{19}[Ff]?|[0-9]{20})$", value):
            raise ValueError(r"must validate the regular expression /^([0-9]{19}[Ff]?|[0-9]{20})$/g")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SimsSearchRequest:
        """Create an instance of SimsSearchRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if iccid (nullable) is None
        # and __fields_set__ contains the field
        if self.iccid is None and "iccid" in self.__fields_set__:
            _dict['iccid'] = None

        # set to None if type (nullable) is None
        # and __fields_set__ contains the field
        if self.type is None and "type" in self.__fields_set__:
            _dict['type'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SimsSearchRequest:
        """Create an instance of SimsSearchRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SimsSearchRequest.parse_obj(obj)

        _obj = SimsSearchRequest.parse_obj({
            "iccid": obj.get("iccid"),
            "type": obj.get("type")
        })
        return _obj




# coding: utf-8

"""
    Gigs Core API

    API to manage voice & data plans for smartphones, wearables and IoT in 200+ countries.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Contact: support@gigs.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Optional
from pydantic import BaseModel, Field, validator

class DeviceModel(BaseModel):
    """
    DeviceModel
    """
    object: Optional[Any] = Field(..., description="Type of object is always `deviceModel`.")
    id: Optional[Any] = Field(..., description="Unique identifier for the deviceModel.")
    brand: Optional[Any] = Field(..., description="The brand of the device model.")
    name: Optional[Any] = Field(..., description="The device model's name.")
    sim_types: Optional[Any] = Field(..., alias="simTypes", description="List of SIM types supported by this device model.")
    type: Optional[Any] = Field(..., description="The type of the device model.")
    __properties = ["object", "id", "brand", "name", "simTypes", "type"]

    @validator('type')
    def type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('car', 'iot', 'laptop', 'router', 'smartphone', 'feature-phone', 'smartwatch', 'tablet', 'wearable', 'other'):
            raise ValueError("must be one of enum values ('car', 'iot', 'laptop', 'router', 'smartphone', 'feature-phone', 'smartwatch', 'tablet', 'wearable', 'other')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> DeviceModel:
        """Create an instance of DeviceModel from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if object (nullable) is None
        # and __fields_set__ contains the field
        if self.object is None and "object" in self.__fields_set__:
            _dict['object'] = None

        # set to None if id (nullable) is None
        # and __fields_set__ contains the field
        if self.id is None and "id" in self.__fields_set__:
            _dict['id'] = None

        # set to None if brand (nullable) is None
        # and __fields_set__ contains the field
        if self.brand is None and "brand" in self.__fields_set__:
            _dict['brand'] = None

        # set to None if name (nullable) is None
        # and __fields_set__ contains the field
        if self.name is None and "name" in self.__fields_set__:
            _dict['name'] = None

        # set to None if sim_types (nullable) is None
        # and __fields_set__ contains the field
        if self.sim_types is None and "sim_types" in self.__fields_set__:
            _dict['simTypes'] = None

        # set to None if type (nullable) is None
        # and __fields_set__ contains the field
        if self.type is None and "type" in self.__fields_set__:
            _dict['type'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> DeviceModel:
        """Create an instance of DeviceModel from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return DeviceModel.parse_obj(obj)

        _obj = DeviceModel.parse_obj({
            "object": obj.get("object"),
            "id": obj.get("id"),
            "brand": obj.get("brand"),
            "name": obj.get("name"),
            "sim_types": obj.get("simTypes"),
            "type": obj.get("type")
        })
        return _obj




# coding: utf-8

"""
    Gigs Core API

    API to manage voice & data plans for smartphones, wearables and IoT in 200+ countries.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Contact: support@gigs.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import re  # noqa: F401
import io
import warnings

from pydantic import validate_arguments, ValidationError
from typing_extensions import Annotated

from pydantic import Field

from typing import Any, Optional

from gigs_client.models.subscription_usage_list200_response import SubscriptionUsageList200Response

from gigs_client.api_client import ApiClient
from gigs_client.api_response import ApiResponse
from gigs_client.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class UsageApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def subscription_usage_list(self, project : Annotated[Any, Field(..., description="The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project).")], id : Annotated[Any, Field(..., description="The unique identifier for the subscription.")], period : Annotated[Optional[Any], Field(description="Limits the usage data returned to the subscription period provided. This option is incompatible with the `start` and `end` parameters.")] = None, start : Annotated[Optional[Any], Field(description="Limits the usage data to dates greater than or equal to the provided date. Can only be used in combination with `end`.")] = None, end : Annotated[Optional[Any], Field(description="Limits the usage data to dates up to and including the provided date. Can only be used in combination with `start`.")] = None, aggregation : Annotated[Optional[Any], Field(description="Determines the aggregation method used, defaulting to `daily`. `period` provides a single aggregated value for the time range or period requested.")] = None, **kwargs) -> SubscriptionUsageList200Response:  # noqa: E501
        """List subscription usage records  # noqa: E501

        Lists usage records in ascending order for a subscription, defaulting to `daily` aggregation for the latest subscription period. If none of the `start`, `end`, or `period` parameters is provided, records are returned for the latest subscription period.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.subscription_usage_list(project, id, period, start, end, aggregation, async_req=True)
        >>> result = thread.get()

        :param project: The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project). (required)
        :type project: object
        :param id: The unique identifier for the subscription. (required)
        :type id: object
        :param period: Limits the usage data returned to the subscription period provided. This option is incompatible with the `start` and `end` parameters.
        :type period: object
        :param start: Limits the usage data to dates greater than or equal to the provided date. Can only be used in combination with `end`.
        :type start: object
        :param end: Limits the usage data to dates up to and including the provided date. Can only be used in combination with `start`.
        :type end: object
        :param aggregation: Determines the aggregation method used, defaulting to `daily`. `period` provides a single aggregated value for the time range or period requested.
        :type aggregation: object
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: SubscriptionUsageList200Response
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the subscription_usage_list_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.subscription_usage_list_with_http_info(project, id, period, start, end, aggregation, **kwargs)  # noqa: E501

    @validate_arguments
    def subscription_usage_list_with_http_info(self, project : Annotated[Any, Field(..., description="The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project).")], id : Annotated[Any, Field(..., description="The unique identifier for the subscription.")], period : Annotated[Optional[Any], Field(description="Limits the usage data returned to the subscription period provided. This option is incompatible with the `start` and `end` parameters.")] = None, start : Annotated[Optional[Any], Field(description="Limits the usage data to dates greater than or equal to the provided date. Can only be used in combination with `end`.")] = None, end : Annotated[Optional[Any], Field(description="Limits the usage data to dates up to and including the provided date. Can only be used in combination with `start`.")] = None, aggregation : Annotated[Optional[Any], Field(description="Determines the aggregation method used, defaulting to `daily`. `period` provides a single aggregated value for the time range or period requested.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """List subscription usage records  # noqa: E501

        Lists usage records in ascending order for a subscription, defaulting to `daily` aggregation for the latest subscription period. If none of the `start`, `end`, or `period` parameters is provided, records are returned for the latest subscription period.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.subscription_usage_list_with_http_info(project, id, period, start, end, aggregation, async_req=True)
        >>> result = thread.get()

        :param project: The unique identifier for the [project](https://developers.gigs.com/docs/api/b3A6MzMwODcxMzI-retrieve-a-project). (required)
        :type project: object
        :param id: The unique identifier for the subscription. (required)
        :type id: object
        :param period: Limits the usage data returned to the subscription period provided. This option is incompatible with the `start` and `end` parameters.
        :type period: object
        :param start: Limits the usage data to dates greater than or equal to the provided date. Can only be used in combination with `end`.
        :type start: object
        :param end: Limits the usage data to dates up to and including the provided date. Can only be used in combination with `start`.
        :type end: object
        :param aggregation: Determines the aggregation method used, defaulting to `daily`. `period` provides a single aggregated value for the time range or period requested.
        :type aggregation: object
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the 
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(SubscriptionUsageList200Response, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'project',
            'id',
            'period',
            'start',
            'end',
            'aggregation'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method subscription_usage_list" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['project']:
            _path_params['project'] = _params['project']

        if _params['id']:
            _path_params['id'] = _params['id']


        # process the query parameters
        _query_params = []
        if _params.get('period') is not None:  # noqa: E501
            _query_params.append(('period', _params['period']))

        if _params.get('start') is not None:  # noqa: E501
            _query_params.append(('start', _params['start']))

        if _params.get('end') is not None:  # noqa: E501
            _query_params.append(('end', _params['end']))

        if _params.get('aggregation') is not None:  # noqa: E501
            _query_params.append(('aggregation', _params['aggregation'].value))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['bearerAuth']  # noqa: E501

        _response_types_map = {
            '200': "SubscriptionUsageList200Response",
            '404': None,
            '422': None,
        }

        return self.api_client.call_api(
            '/projects/{project}/subscriptions/{id}/usage', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# coding: utf-8

"""
    Gigs Core API

    API to manage voice & data plans for smartphones, wearables and IoT in 200+ countries.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Contact: support@gigs.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Optional
from pydantic import BaseModel, Field, validator
from gigs_client.models.subscription_change_plan import SubscriptionChangePlan

class SubscriptionChange(BaseModel):
    """
    SubscriptionChange
    """
    object: Optional[Any] = Field(..., description="Type of object is always `subscriptionChange`.")
    id: Optional[Any] = Field(..., description="Unique identifier for the subscription change.")
    plan: SubscriptionChangePlan = Field(...)
    status: Optional[Any] = Field(..., description="The current status of the subscription change.")
    subscription: Optional[Any] = Field(..., description="Unique identifier of the subscription to be changed.")
    applied_at: Optional[Any] = Field(..., alias="appliedAt", description="Time when the subscription change was applied.")
    created_at: Optional[Any] = Field(..., alias="createdAt", description="Time when the subscription change was created.")
    scheduled_at: Optional[Any] = Field(..., alias="scheduledAt", description="Approximate time when the change should take place.")
    __properties = ["object", "id", "plan", "status", "subscription", "appliedAt", "createdAt", "scheduledAt"]

    @validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('pending', 'applied'):
            raise ValueError("must be one of enum values ('pending', 'applied')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SubscriptionChange:
        """Create an instance of SubscriptionChange from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of plan
        if self.plan:
            _dict['plan'] = self.plan.to_dict()
        # set to None if object (nullable) is None
        # and __fields_set__ contains the field
        if self.object is None and "object" in self.__fields_set__:
            _dict['object'] = None

        # set to None if id (nullable) is None
        # and __fields_set__ contains the field
        if self.id is None and "id" in self.__fields_set__:
            _dict['id'] = None

        # set to None if status (nullable) is None
        # and __fields_set__ contains the field
        if self.status is None and "status" in self.__fields_set__:
            _dict['status'] = None

        # set to None if subscription (nullable) is None
        # and __fields_set__ contains the field
        if self.subscription is None and "subscription" in self.__fields_set__:
            _dict['subscription'] = None

        # set to None if applied_at (nullable) is None
        # and __fields_set__ contains the field
        if self.applied_at is None and "applied_at" in self.__fields_set__:
            _dict['appliedAt'] = None

        # set to None if created_at (nullable) is None
        # and __fields_set__ contains the field
        if self.created_at is None and "created_at" in self.__fields_set__:
            _dict['createdAt'] = None

        # set to None if scheduled_at (nullable) is None
        # and __fields_set__ contains the field
        if self.scheduled_at is None and "scheduled_at" in self.__fields_set__:
            _dict['scheduledAt'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SubscriptionChange:
        """Create an instance of SubscriptionChange from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SubscriptionChange.parse_obj(obj)

        _obj = SubscriptionChange.parse_obj({
            "object": obj.get("object"),
            "id": obj.get("id"),
            "plan": SubscriptionChangePlan.from_dict(obj.get("plan")) if obj.get("plan") is not None else None,
            "status": obj.get("status"),
            "subscription": obj.get("subscription"),
            "applied_at": obj.get("appliedAt"),
            "created_at": obj.get("createdAt"),
            "scheduled_at": obj.get("scheduledAt")
        })
        return _obj




# coding: utf-8

"""
    Gigs Core API

    API to manage voice & data plans for smartphones, wearables and IoT in 200+ countries.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Contact: support@gigs.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Optional
from pydantic import BaseModel, Field

class ProjectLegal(BaseModel):
    """
    The legal address for the company that manages the project.
    """
    name: Optional[Any] = Field(..., description="The company's legal name.")
    city: Optional[Any] = Field(..., description="The city/municipality of the company's legal address.")
    country: Optional[Any] = Field(..., description="The ISO 3166-1 alpha-2 country code for the company's legal address.")
    line1: Optional[Any] = Field(..., description="The first line of the company's legal address, e.g. street and house number.")
    line2: Optional[Any] = Field(..., description="The second line of the given street address, e.g. Apartment number.")
    postal_code: Optional[Any] = Field(..., alias="postalCode", description="The postal code of the given street address.")
    state: Optional[Any] = Field(..., description="The state/province/region of the given street address.")
    __properties = ["name", "city", "country", "line1", "line2", "postalCode", "state"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ProjectLegal:
        """Create an instance of ProjectLegal from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if name (nullable) is None
        # and __fields_set__ contains the field
        if self.name is None and "name" in self.__fields_set__:
            _dict['name'] = None

        # set to None if city (nullable) is None
        # and __fields_set__ contains the field
        if self.city is None and "city" in self.__fields_set__:
            _dict['city'] = None

        # set to None if country (nullable) is None
        # and __fields_set__ contains the field
        if self.country is None and "country" in self.__fields_set__:
            _dict['country'] = None

        # set to None if line1 (nullable) is None
        # and __fields_set__ contains the field
        if self.line1 is None and "line1" in self.__fields_set__:
            _dict['line1'] = None

        # set to None if line2 (nullable) is None
        # and __fields_set__ contains the field
        if self.line2 is None and "line2" in self.__fields_set__:
            _dict['line2'] = None

        # set to None if postal_code (nullable) is None
        # and __fields_set__ contains the field
        if self.postal_code is None and "postal_code" in self.__fields_set__:
            _dict['postalCode'] = None

        # set to None if state (nullable) is None
        # and __fields_set__ contains the field
        if self.state is None and "state" in self.__fields_set__:
            _dict['state'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ProjectLegal:
        """Create an instance of ProjectLegal from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ProjectLegal.parse_obj(obj)

        _obj = ProjectLegal.parse_obj({
            "name": obj.get("name"),
            "city": obj.get("city"),
            "country": obj.get("country"),
            "line1": obj.get("line1"),
            "line2": obj.get("line2"),
            "postal_code": obj.get("postalCode"),
            "state": obj.get("state")
        })
        return _obj




# coding: utf-8

"""
    Gigs Core API

    API to manage voice & data plans for smartphones, wearables and IoT in 200+ countries.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Contact: support@gigs.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Optional
from pydantic import BaseModel, Field

class UsersCreateRequest(BaseModel):
    """
    UsersCreateRequest
    """
    birthday: Optional[Any] = Field(None, description="The birthday of the user.")
    email: Optional[Any] = Field(..., description="The primary verified email address of the user.")
    full_name: Optional[Any] = Field(None, alias="fullName", description="The user's full name. Some plans require the user name to be present when creating a subscription. Check the plan requirements for that.")
    preferred_locale: Optional[Any] = Field(None, alias="preferredLocale", description="The user's locale preference represented as an [IETF language tag](https://en.wikipedia.org/wiki/IETF_language_tag). It defaults to the project's preferred locale.")
    __properties = ["birthday", "email", "fullName", "preferredLocale"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> UsersCreateRequest:
        """Create an instance of UsersCreateRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # set to None if birthday (nullable) is None
        # and __fields_set__ contains the field
        if self.birthday is None and "birthday" in self.__fields_set__:
            _dict['birthday'] = None

        # set to None if email (nullable) is None
        # and __fields_set__ contains the field
        if self.email is None and "email" in self.__fields_set__:
            _dict['email'] = None

        # set to None if full_name (nullable) is None
        # and __fields_set__ contains the field
        if self.full_name is None and "full_name" in self.__fields_set__:
            _dict['fullName'] = None

        # set to None if preferred_locale (nullable) is None
        # and __fields_set__ contains the field
        if self.preferred_locale is None and "preferred_locale" in self.__fields_set__:
            _dict['preferredLocale'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> UsersCreateRequest:
        """Create an instance of UsersCreateRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return UsersCreateRequest.parse_obj(obj)

        _obj = UsersCreateRequest.parse_obj({
            "birthday": obj.get("birthday"),
            "email": obj.get("email"),
            "full_name": obj.get("fullName"),
            "preferred_locale": obj.get("preferredLocale")
        })
        return _obj




# coding: utf-8

"""
    Gigs Core API

    API to manage voice & data plans for smartphones, wearables and IoT in 200+ countries.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Contact: support@gigs.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Optional
from pydantic import BaseModel, Field, validator
from gigs_client.models.subscription_addon_addon import SubscriptionAddonAddon
from gigs_client.models.subscription_addon_current_period import SubscriptionAddonCurrentPeriod

class SubscriptionAddon(BaseModel):
    """
    <!-- theme: info --> > #### Preview > > This schema is currently in preview and might change in the future. > We’re excited to hear your feedback and ideas. Please send an email > to [support@gigs.com](mailto:support@gigs.com) to share your thoughts. 
    """
    object: Optional[Any] = Field(..., description="Type of object is always `subscriptionAddon`.")
    id: Optional[Any] = Field(..., description="Unique identifier for the subscription addon.")
    addon: SubscriptionAddonAddon = Field(...)
    current_period: SubscriptionAddonCurrentPeriod = Field(..., alias="currentPeriod")
    status: Optional[Any] = Field(..., description="The current status of the subscription addon.")
    subscription: Optional[Any] = Field(..., description="Unique identifier of the subscription the addon applies to.")
    user: Optional[Any] = Field(..., description="Unique identifier of the user the subscription addon belongs to.")
    activated_at: Optional[Any] = Field(..., alias="activatedAt", description="Time when the subscription addon was activated.")
    canceled_at: Optional[Any] = Field(..., alias="canceledAt", description="Time when the subscription addon was canceled.")
    created_at: Optional[Any] = Field(..., alias="createdAt", description="Time when the subscription addon was created.")
    ended_at: Optional[Any] = Field(..., alias="endedAt", description="Time when the subscription addon was ended.")
    __properties = ["object", "id", "addon", "currentPeriod", "status", "subscription", "user", "activatedAt", "canceledAt", "createdAt", "endedAt"]

    @validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in ('pending', 'active', 'ended'):
            raise ValueError("must be one of enum values ('pending', 'active', 'ended')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SubscriptionAddon:
        """Create an instance of SubscriptionAddon from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of addon
        if self.addon:
            _dict['addon'] = self.addon.to_dict()
        # override the default output from pydantic by calling `to_dict()` of current_period
        if self.current_period:
            _dict['currentPeriod'] = self.current_period.to_dict()
        # set to None if object (nullable) is None
        # and __fields_set__ contains the field
        if self.object is None and "object" in self.__fields_set__:
            _dict['object'] = None

        # set to None if id (nullable) is None
        # and __fields_set__ contains the field
        if self.id is None and "id" in self.__fields_set__:
            _dict['id'] = None

        # set to None if status (nullable) is None
        # and __fields_set__ contains the field
        if self.status is None and "status" in self.__fields_set__:
            _dict['status'] = None

        # set to None if subscription (nullable) is None
        # and __fields_set__ contains the field
        if self.subscription is None and "subscription" in self.__fields_set__:
            _dict['subscription'] = None

        # set to None if user (nullable) is None
        # and __fields_set__ contains the field
        if self.user is None and "user" in self.__fields_set__:
            _dict['user'] = None

        # set to None if activated_at (nullable) is None
        # and __fields_set__ contains the field
        if self.activated_at is None and "activated_at" in self.__fields_set__:
            _dict['activatedAt'] = None

        # set to None if canceled_at (nullable) is None
        # and __fields_set__ contains the field
        if self.canceled_at is None and "canceled_at" in self.__fields_set__:
            _dict['canceledAt'] = None

        # set to None if created_at (nullable) is None
        # and __fields_set__ contains the field
        if self.created_at is None and "created_at" in self.__fields_set__:
            _dict['createdAt'] = None

        # set to None if ended_at (nullable) is None
        # and __fields_set__ contains the field
        if self.ended_at is None and "ended_at" in self.__fields_set__:
            _dict['endedAt'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SubscriptionAddon:
        """Create an instance of SubscriptionAddon from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SubscriptionAddon.parse_obj(obj)

        _obj = SubscriptionAddon.parse_obj({
            "object": obj.get("object"),
            "id": obj.get("id"),
            "addon": SubscriptionAddonAddon.from_dict(obj.get("addon")) if obj.get("addon") is not None else None,
            "current_period": SubscriptionAddonCurrentPeriod.from_dict(obj.get("currentPeriod")) if obj.get("currentPeriod") is not None else None,
            "status": obj.get("status"),
            "subscription": obj.get("subscription"),
            "user": obj.get("user"),
            "activated_at": obj.get("activatedAt"),
            "canceled_at": obj.get("canceledAt"),
            "created_at": obj.get("createdAt"),
            "ended_at": obj.get("endedAt")
        })
        return _obj




# coding: utf-8

"""
    Gigs Core API

    API to manage voice & data plans for smartphones, wearables and IoT in 200+ countries.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Contact: support@gigs.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Optional
from pydantic import BaseModel, Field
from gigs_client.models.plans_search_request_compatible_with import PlansSearchRequestCompatibleWith

class PlansSearchRequest(BaseModel):
    """
    PlansSearchRequest
    """
    status: Optional[Any] = Field(None, description="The status to filter plans by.")
    compatible_with: PlansSearchRequestCompatibleWith = Field(..., alias="compatibleWith")
    __properties = ["status", "compatibleWith"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PlansSearchRequest:
        """Create an instance of PlansSearchRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of compatible_with
        if self.compatible_with:
            _dict['compatibleWith'] = self.compatible_with.to_dict()
        # set to None if status (nullable) is None
        # and __fields_set__ contains the field
        if self.status is None and "status" in self.__fields_set__:
            _dict['status'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> PlansSearchRequest:
        """Create an instance of PlansSearchRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return PlansSearchRequest.parse_obj(obj)

        _obj = PlansSearchRequest.parse_obj({
            "status": obj.get("status"),
            "compatible_with": PlansSearchRequestCompatibleWith.from_dict(obj.get("compatibleWith")) if obj.get("compatibleWith") is not None else None
        })
        return _obj

